<?php

namespace FiloBlu\Flow\Model\From;

use Magento\CatalogInventory\Model\Stock as CatalogInventoryStock;

/**
 * Class Stock
 * @package FiloBlu\Flow\Model\From
 */
class Stock extends AbstractFrom
{

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;
    /**
     * @var \FiloBlu\Flow\Helper\Data
     */
    protected $_helper_flow;
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_product_factory;
    /**
     * @var \Magento\CatalogInventory\Api\StockRegistryInterface
     */
    protected $stockRegistry;
    /**
     * @var \FiloBlu\Flow\Helper\Stock
     */
    protected $_stock_qty;
    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $_productMetadata;

    /**
     * @var \FiloBlu\Flow\Helper\EmailNotification
     */
    protected $_emailnotification_helper;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * Stock constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \FiloBlu\Flow\Helper\Data $helper_flow
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
     * @param \FiloBlu\Flow\Helper\Stock $stock_qty
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \FiloBlu\Flow\Helper\Stock $stock_qty,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \FiloBlu\Flow\Helper\EmailNotification $emailnotification_helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        array $data = []
    )
    {
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        $this->_product_factory = $productFactory;
        $this->stockRegistry = $stockRegistry;
        $this->_stock_qty = $stock_qty;
        $this->_productMetadata = $productMetadata;
        $this->_emailnotification_helper = $emailnotification_helper;
        $this->_scopeConfig = $scopeConfig;
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    }

    /**
     * @return $this|bool|mixed
     * @throws \Exception
     */
    public function process()
    {

        $sku = $this->getSku();
        $product_id = $this->_product_factory->create()->getIdBySku($sku);

        if (!$product_id) {
            return $this->error(false, "Product not found! product_code: '{$sku}'");
        }

        $stock = $this->stockRegistry->getStockItem($product_id);

        try {

            $raw_qty = intval($this->getData('fisico_disponibile'));
            $qty = $this->_stock_qty->getRealQty($product_id, $raw_qty);

            // Se non viene ritornata una quantità numerica ma una stringa vuol dire che c'è stato un errore
            if (!is_numeric($qty)) {
                // Salvo il messaggio relativo all'errore
                $this->addEmailNotifyError($qty);
                // Dopo aver salvato il messaggio aggiorno la quantità da impostare a 0
                $qty = 0;
            }

            $order_list = $this->_stock_qty->getReservedQtyOrderList($product_id);

            $stock->setQty($qty);

            /* Leave the stock  inventory status untouched for products having backorder flag set */

            if ($stock->getBackorders() == CatalogInventoryStock::BACKORDERS_NO) {
                if ($qty > $stock->getMinQty()) {
                    $stock->setIsInStock(true);
                } else {
                    $stock->setIsInStock(false);
                }
            }

            $stock_data = $stock->getData();

            if (isset($stock_data['type_id'])) {
                unset($stock_data['type_id']);
            }

            if (isset($stock_data['stock_status_changed_automatically_flag'])) {
                unset($stock_data['stock_status_changed_automatically_flag']);
            }

            $this->getResource()->getConnection()->insertOnDuplicate('cataloginventory_stock_item', $stock_data);

            if ($raw_qty != $qty) {
                $preserved_qty = $raw_qty - $qty;
                $this->error(false, "Preserved Quantity '{$preserved_qty}' For Product ID '{$product_id}' SKU '{$sku}' (From orders: '{$order_list}')");
            }

            $this->_logger->info("Process STOCK - qty {$qty}  ID: {$product_id}  SKU: {$sku}");

        } catch (\Exception $e) {
            $this->_logger->info('Errore nel salvataggio dello Stock. ' . $e->getMessage());
        }

        return $this;
    }

    /**
     * @param $file
     * @return mixed|void
     * @throws \Magento\Framework\Exception\MailException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function sendErrorNotifications($file)
    {

        // Se sono stati rilevati errori e salvati i relativi messaggi si procede con la composizione della mail
        if (!empty($this->getEmailNotifyError())) {

            // Recupero gli indirizzi email dei destinatari
            $emailAddressString = $this->_scopeConfig->getValue(self::XML_PATH_EMAIL_ADDRESS_NOTIFY_ERROR);

            if ($emailAddressString && $emailAddressString != '') {
                $emailAddress = explode(',', $emailAddressString);
                $emailAddress = array_map('trim', $emailAddress);

                $storeName = $this->_scopeConfig->getValue(self::XML_PATH_STORE_NAME);
                // Compongo il testo del messaggio con i vari errori
                $errorString = "<p>" . implode("</p><p>", $this->getEmailNotifyError()) . "</p>";

                // Array con i dati che serviranno a comporre la mail
                $data = array(
                    'mail_subject' => 'Reporting breakdowns in stock loading for ' . $storeName,
                    'file_name' => $file->getName(),
                    'errors' => $errorString
                );

                // Invio email di notifica
                $this->_emailnotification_helper->notify($emailAddress, $data);
            }
        }

    }

    /**
     * @param $query
     * @return bool
     */
    protected function postprocessFileRows($query)
    {

        try {

            $connection = $this->getResource()->getConnection();

            $entityField = $this->getEntityField();

            /* Find all configurables not in backorders and having child quantity sum greater than 0*/
            $configurablesIdsSql = "SELECT
	entity_id
FROM
	catalog_product_entity
WHERE
	type_id = 'configurable'
    AND {$entityField} IN (
                SELECT
		cpsl.parent_id
	FROM
		cataloginventory_stock_item AS csi
	LEFT JOIN catalog_product_entity AS cpe ON
		cpe.entity_id = csi.product_id
	LEFT JOIN catalog_product_super_link AS cpsl ON
		cpe.{$entityField} = cpsl.product_id
	WHERE
		csi.use_config_backorders = 0
        and cpsl.parent_id is not null
	GROUP BY
		cpsl.parent_id
	HAVING
		SUM(csi.qty) > 0 )";

            $configurableIds = [];

            foreach ($connection->fetchAll($configurablesIdsSql, \Zend_Db::FETCH_NUM) AS $row) {
                $configurableIds[] = $row['entity_id'];
            }

            $ids = join(',', $configurableIds);
            $connection->query("UPDATE cataloginventory_stock_item SET is_in_stock  = 1 WHERE `product_id` IN ({$ids})");

        } catch (\Exception $e) {
            $this->_logger->info('Errore nel salvataggio del Post Stock Item Fix Tuning. ' . $e->getMessage());
            return false;
        }

        return true;

    }

    /**
     * @return string
     */
    protected function getEntityField()
    {
        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';
    }
}